# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TaskTableConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'task_id': 'str',
        'function_name': 'str',
        'current_state': 'list[TaskState]',
        'text': 'str',
        'job_id': 'str',
        'exception_type': 'str',
        'attempts': 'TaskAttempts',
        'data_operator_id': 'str',
        'latest_attempt_only': 'bool',
        'node_id': 'str',
        'actor_id': 'str'
    }

    attribute_map = {
        'task_id': 'task_id',
        'function_name': 'function_name',
        'current_state': 'current_state',
        'text': 'text',
        'job_id': 'job_id',
        'exception_type': 'exception_type',
        'attempts': 'attempts',
        'data_operator_id': 'data_operator_id',
        'latest_attempt_only': 'latest_attempt_only',
        'node_id': 'node_id',
        'actor_id': 'actor_id'
    }

    def __init__(self, task_id=None, function_name=None, current_state=None, text=None, job_id=None, exception_type=None, attempts=None, data_operator_id=None, latest_attempt_only=None, node_id=None, actor_id=None, local_vars_configuration=None):  # noqa: E501
        """TaskTableConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._task_id = None
        self._function_name = None
        self._current_state = None
        self._text = None
        self._job_id = None
        self._exception_type = None
        self._attempts = None
        self._data_operator_id = None
        self._latest_attempt_only = None
        self._node_id = None
        self._actor_id = None
        self.discriminator = None

        if task_id is not None:
            self.task_id = task_id
        if function_name is not None:
            self.function_name = function_name
        if current_state is not None:
            self.current_state = current_state
        if text is not None:
            self.text = text
        if job_id is not None:
            self.job_id = job_id
        if exception_type is not None:
            self.exception_type = exception_type
        if attempts is not None:
            self.attempts = attempts
        if data_operator_id is not None:
            self.data_operator_id = data_operator_id
        if latest_attempt_only is not None:
            self.latest_attempt_only = latest_attempt_only
        if node_id is not None:
            self.node_id = node_id
        if actor_id is not None:
            self.actor_id = actor_id

    @property
    def task_id(self):
        """Gets the task_id of this TaskTableConfig.  # noqa: E501


        :return: The task_id of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._task_id

    @task_id.setter
    def task_id(self, task_id):
        """Sets the task_id of this TaskTableConfig.


        :param task_id: The task_id of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._task_id = task_id

    @property
    def function_name(self):
        """Gets the function_name of this TaskTableConfig.  # noqa: E501


        :return: The function_name of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._function_name

    @function_name.setter
    def function_name(self, function_name):
        """Sets the function_name of this TaskTableConfig.


        :param function_name: The function_name of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._function_name = function_name

    @property
    def current_state(self):
        """Gets the current_state of this TaskTableConfig.  # noqa: E501


        :return: The current_state of this TaskTableConfig.  # noqa: E501
        :rtype: list[TaskState]
        """
        return self._current_state

    @current_state.setter
    def current_state(self, current_state):
        """Sets the current_state of this TaskTableConfig.


        :param current_state: The current_state of this TaskTableConfig.  # noqa: E501
        :type: list[TaskState]
        """

        self._current_state = current_state

    @property
    def text(self):
        """Gets the text of this TaskTableConfig.  # noqa: E501


        :return: The text of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._text

    @text.setter
    def text(self, text):
        """Sets the text of this TaskTableConfig.


        :param text: The text of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._text = text

    @property
    def job_id(self):
        """Gets the job_id of this TaskTableConfig.  # noqa: E501


        :return: The job_id of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this TaskTableConfig.


        :param job_id: The job_id of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._job_id = job_id

    @property
    def exception_type(self):
        """Gets the exception_type of this TaskTableConfig.  # noqa: E501


        :return: The exception_type of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._exception_type

    @exception_type.setter
    def exception_type(self, exception_type):
        """Sets the exception_type of this TaskTableConfig.


        :param exception_type: The exception_type of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._exception_type = exception_type

    @property
    def attempts(self):
        """Gets the attempts of this TaskTableConfig.  # noqa: E501


        :return: The attempts of this TaskTableConfig.  # noqa: E501
        :rtype: TaskAttempts
        """
        return self._attempts

    @attempts.setter
    def attempts(self, attempts):
        """Sets the attempts of this TaskTableConfig.


        :param attempts: The attempts of this TaskTableConfig.  # noqa: E501
        :type: TaskAttempts
        """

        self._attempts = attempts

    @property
    def data_operator_id(self):
        """Gets the data_operator_id of this TaskTableConfig.  # noqa: E501


        :return: The data_operator_id of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._data_operator_id

    @data_operator_id.setter
    def data_operator_id(self, data_operator_id):
        """Sets the data_operator_id of this TaskTableConfig.


        :param data_operator_id: The data_operator_id of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._data_operator_id = data_operator_id

    @property
    def latest_attempt_only(self):
        """Gets the latest_attempt_only of this TaskTableConfig.  # noqa: E501


        :return: The latest_attempt_only of this TaskTableConfig.  # noqa: E501
        :rtype: bool
        """
        return self._latest_attempt_only

    @latest_attempt_only.setter
    def latest_attempt_only(self, latest_attempt_only):
        """Sets the latest_attempt_only of this TaskTableConfig.


        :param latest_attempt_only: The latest_attempt_only of this TaskTableConfig.  # noqa: E501
        :type: bool
        """

        self._latest_attempt_only = latest_attempt_only

    @property
    def node_id(self):
        """Gets the node_id of this TaskTableConfig.  # noqa: E501


        :return: The node_id of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._node_id

    @node_id.setter
    def node_id(self, node_id):
        """Sets the node_id of this TaskTableConfig.


        :param node_id: The node_id of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._node_id = node_id

    @property
    def actor_id(self):
        """Gets the actor_id of this TaskTableConfig.  # noqa: E501


        :return: The actor_id of this TaskTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._actor_id

    @actor_id.setter
    def actor_id(self, actor_id):
        """Sets the actor_id of this TaskTableConfig.


        :param actor_id: The actor_id of this TaskTableConfig.  # noqa: E501
        :type: str
        """

        self._actor_id = actor_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TaskTableConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TaskTableConfig):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class SSOConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'sso_connection': 'SSOConnection',
        'directory_sync': 'DirectorySync',
        'domain_verifications': 'list[DomainVerification]'
    }

    attribute_map = {
        'sso_connection': 'sso_connection',
        'directory_sync': 'directory_sync',
        'domain_verifications': 'domain_verifications'
    }

    def __init__(self, sso_connection=None, directory_sync=None, domain_verifications=None, local_vars_configuration=None):  # noqa: E501
        """SSOConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._sso_connection = None
        self._directory_sync = None
        self._domain_verifications = None
        self.discriminator = None

        self.sso_connection = sso_connection
        self.directory_sync = directory_sync
        self.domain_verifications = domain_verifications

    @property
    def sso_connection(self):
        """Gets the sso_connection of this SSOConfig.  # noqa: E501


        :return: The sso_connection of this SSOConfig.  # noqa: E501
        :rtype: SSOConnection
        """
        return self._sso_connection

    @sso_connection.setter
    def sso_connection(self, sso_connection):
        """Sets the sso_connection of this SSOConfig.


        :param sso_connection: The sso_connection of this SSOConfig.  # noqa: E501
        :type: SSOConnection
        """
        if self.local_vars_configuration.client_side_validation and sso_connection is None:  # noqa: E501
            raise ValueError("Invalid value for `sso_connection`, must not be `None`")  # noqa: E501

        self._sso_connection = sso_connection

    @property
    def directory_sync(self):
        """Gets the directory_sync of this SSOConfig.  # noqa: E501


        :return: The directory_sync of this SSOConfig.  # noqa: E501
        :rtype: DirectorySync
        """
        return self._directory_sync

    @directory_sync.setter
    def directory_sync(self, directory_sync):
        """Sets the directory_sync of this SSOConfig.


        :param directory_sync: The directory_sync of this SSOConfig.  # noqa: E501
        :type: DirectorySync
        """
        if self.local_vars_configuration.client_side_validation and directory_sync is None:  # noqa: E501
            raise ValueError("Invalid value for `directory_sync`, must not be `None`")  # noqa: E501

        self._directory_sync = directory_sync

    @property
    def domain_verifications(self):
        """Gets the domain_verifications of this SSOConfig.  # noqa: E501


        :return: The domain_verifications of this SSOConfig.  # noqa: E501
        :rtype: list[DomainVerification]
        """
        return self._domain_verifications

    @domain_verifications.setter
    def domain_verifications(self, domain_verifications):
        """Sets the domain_verifications of this SSOConfig.


        :param domain_verifications: The domain_verifications of this SSOConfig.  # noqa: E501
        :type: list[DomainVerification]
        """
        if self.local_vars_configuration.client_side_validation and domain_verifications is None:  # noqa: E501
            raise ValueError("Invalid value for `domain_verifications`, must not be `None`")  # noqa: E501

        self._domain_verifications = domain_verifications

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SSOConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SSOConfig):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ConnectionVerificationResult(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'connection_id': 'str',
        'is_healthy': 'bool',
        'verified_at': 'datetime',
        'details': 'str',
        'error_message': 'str'
    }

    attribute_map = {
        'connection_id': 'connection_id',
        'is_healthy': 'is_healthy',
        'verified_at': 'verified_at',
        'details': 'details',
        'error_message': 'error_message'
    }

    def __init__(self, connection_id=None, is_healthy=None, verified_at=None, details=None, error_message=None, local_vars_configuration=None):  # noqa: E501
        """ConnectionVerificationResult - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._connection_id = None
        self._is_healthy = None
        self._verified_at = None
        self._details = None
        self._error_message = None
        self.discriminator = None

        self.connection_id = connection_id
        self.is_healthy = is_healthy
        self.verified_at = verified_at
        if details is not None:
            self.details = details
        if error_message is not None:
            self.error_message = error_message

    @property
    def connection_id(self):
        """Gets the connection_id of this ConnectionVerificationResult.  # noqa: E501

        ID of the verified connection  # noqa: E501

        :return: The connection_id of this ConnectionVerificationResult.  # noqa: E501
        :rtype: str
        """
        return self._connection_id

    @connection_id.setter
    def connection_id(self, connection_id):
        """Sets the connection_id of this ConnectionVerificationResult.

        ID of the verified connection  # noqa: E501

        :param connection_id: The connection_id of this ConnectionVerificationResult.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and connection_id is None:  # noqa: E501
            raise ValueError("Invalid value for `connection_id`, must not be `None`")  # noqa: E501

        self._connection_id = connection_id

    @property
    def is_healthy(self):
        """Gets the is_healthy of this ConnectionVerificationResult.  # noqa: E501

        Whether the connection is currently healthy and accessible  # noqa: E501

        :return: The is_healthy of this ConnectionVerificationResult.  # noqa: E501
        :rtype: bool
        """
        return self._is_healthy

    @is_healthy.setter
    def is_healthy(self, is_healthy):
        """Sets the is_healthy of this ConnectionVerificationResult.

        Whether the connection is currently healthy and accessible  # noqa: E501

        :param is_healthy: The is_healthy of this ConnectionVerificationResult.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and is_healthy is None:  # noqa: E501
            raise ValueError("Invalid value for `is_healthy`, must not be `None`")  # noqa: E501

        self._is_healthy = is_healthy

    @property
    def verified_at(self):
        """Gets the verified_at of this ConnectionVerificationResult.  # noqa: E501

        Timestamp when verification was performed  # noqa: E501

        :return: The verified_at of this ConnectionVerificationResult.  # noqa: E501
        :rtype: datetime
        """
        return self._verified_at

    @verified_at.setter
    def verified_at(self, verified_at):
        """Sets the verified_at of this ConnectionVerificationResult.

        Timestamp when verification was performed  # noqa: E501

        :param verified_at: The verified_at of this ConnectionVerificationResult.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and verified_at is None:  # noqa: E501
            raise ValueError("Invalid value for `verified_at`, must not be `None`")  # noqa: E501

        self._verified_at = verified_at

    @property
    def details(self):
        """Gets the details of this ConnectionVerificationResult.  # noqa: E501

        Additional details about verification success (e.g., catalogs found)  # noqa: E501

        :return: The details of this ConnectionVerificationResult.  # noqa: E501
        :rtype: str
        """
        return self._details

    @details.setter
    def details(self, details):
        """Sets the details of this ConnectionVerificationResult.

        Additional details about verification success (e.g., catalogs found)  # noqa: E501

        :param details: The details of this ConnectionVerificationResult.  # noqa: E501
        :type: str
        """

        self._details = details

    @property
    def error_message(self):
        """Gets the error_message of this ConnectionVerificationResult.  # noqa: E501

        Error message if verification failed  # noqa: E501

        :return: The error_message of this ConnectionVerificationResult.  # noqa: E501
        :rtype: str
        """
        return self._error_message

    @error_message.setter
    def error_message(self, error_message):
        """Sets the error_message of this ConnectionVerificationResult.

        Error message if verification failed  # noqa: E501

        :param error_message: The error_message of this ConnectionVerificationResult.  # noqa: E501
        :type: str
        """

        self._error_message = error_message

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConnectionVerificationResult):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ConnectionVerificationResult):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ListResourceQuotasQuery(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'TextQuery',
        'cloud_id': 'str',
        'creator_id': 'str',
        'is_enabled': 'bool',
        'paging': 'PageQuery'
    }

    attribute_map = {
        'name': 'name',
        'cloud_id': 'cloud_id',
        'creator_id': 'creator_id',
        'is_enabled': 'is_enabled',
        'paging': 'paging'
    }

    def __init__(self, name=None, cloud_id=None, creator_id=None, is_enabled=None, paging=None, local_vars_configuration=None):  # noqa: E501
        """ListResourceQuotasQuery - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._cloud_id = None
        self._creator_id = None
        self._is_enabled = None
        self._paging = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if cloud_id is not None:
            self.cloud_id = cloud_id
        if creator_id is not None:
            self.creator_id = creator_id
        if is_enabled is not None:
            self.is_enabled = is_enabled
        if paging is not None:
            self.paging = paging

    @property
    def name(self):
        """Gets the name of this ListResourceQuotasQuery.  # noqa: E501

        The name filter for the resource quotas.  # noqa: E501

        :return: The name of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: TextQuery
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ListResourceQuotasQuery.

        The name filter for the resource quotas.  # noqa: E501

        :param name: The name of this ListResourceQuotasQuery.  # noqa: E501
        :type: TextQuery
        """

        self._name = name

    @property
    def cloud_id(self):
        """Gets the cloud_id of this ListResourceQuotasQuery.  # noqa: E501

        The cloud ID filter for the resource quotas.  # noqa: E501

        :return: The cloud_id of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this ListResourceQuotasQuery.

        The cloud ID filter for the resource quotas.  # noqa: E501

        :param cloud_id: The cloud_id of this ListResourceQuotasQuery.  # noqa: E501
        :type: str
        """

        self._cloud_id = cloud_id

    @property
    def creator_id(self):
        """Gets the creator_id of this ListResourceQuotasQuery.  # noqa: E501

        The creator ID filter for the resource quotas.  # noqa: E501

        :return: The creator_id of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: str
        """
        return self._creator_id

    @creator_id.setter
    def creator_id(self, creator_id):
        """Sets the creator_id of this ListResourceQuotasQuery.

        The creator ID filter for the resource quotas.  # noqa: E501

        :param creator_id: The creator_id of this ListResourceQuotasQuery.  # noqa: E501
        :type: str
        """

        self._creator_id = creator_id

    @property
    def is_enabled(self):
        """Gets the is_enabled of this ListResourceQuotasQuery.  # noqa: E501

        The is_enabled filter for the resource quotas.  # noqa: E501

        :return: The is_enabled of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: bool
        """
        return self._is_enabled

    @is_enabled.setter
    def is_enabled(self, is_enabled):
        """Sets the is_enabled of this ListResourceQuotasQuery.

        The is_enabled filter for the resource quotas.  # noqa: E501

        :param is_enabled: The is_enabled of this ListResourceQuotasQuery.  # noqa: E501
        :type: bool
        """

        self._is_enabled = is_enabled

    @property
    def paging(self):
        """Gets the paging of this ListResourceQuotasQuery.  # noqa: E501

        Pagination information.  # noqa: E501

        :return: The paging of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: PageQuery
        """
        return self._paging

    @paging.setter
    def paging(self, paging):
        """Sets the paging of this ListResourceQuotasQuery.

        Pagination information.  # noqa: E501

        :param paging: The paging of this ListResourceQuotasQuery.  # noqa: E501
        :type: PageQuery
        """

        self._paging = paging

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ListResourceQuotasQuery):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ListResourceQuotasQuery):
            return True

        return self.to_dict() != other.to_dict()



from dataclasses import dataclass, field
from typing import ClassVar, Dict, List

from anyscale._private.models import ModelBase, ModelEnum


class PolicySyncStatus(ModelEnum):
    """Sync status for resource permission policies."""

    pending = "pending"
    success = "success"
    failed = "failed"

    __docstrings__: ClassVar[Dict[str, str]] = {
        "pending": "Policy is pending synchronization.",
        "success": "Policy has been successfully synchronized.",
        "failed": "Policy synchronization has failed.",
    }


@dataclass(frozen=True)
class PolicyBinding(ModelBase):
    """A binding of a role to a list of principals (user group IDs)."""

    __doc_py_example__ = """\
from anyscale.policy.models import PolicyBinding

binding = PolicyBinding(role_name="collaborator", principals=["ug_abc123"])
"""

    role_name: str = field(
        metadata={
            "docstring": (
                "The role name. For cloud/project policies use 'collaborator' or "
                "'readonly'. For organization "
                "policies use 'owner' or 'collaborator'."
            )
        }
    )

    def _validate_role_name(self, role_name: str) -> str:
        if not isinstance(role_name, str):
            raise TypeError("role_name must be a string.")
        normalized = role_name.strip().lower()
        if normalized == "write":
            normalized = "collaborator"
        allowed = {"owner", "collaborator", "readonly"}
        if normalized not in allowed:
            raise ValueError(
                f"Invalid role_name '{role_name}'. Allowed values: "
                f"{', '.join(sorted(allowed))}."
            )
        return normalized

    principals: List[str] = field(
        metadata={"docstring": "List of user group IDs that have this role."}
    )

    def _validate_principals(self, principals: List[str]):
        if not isinstance(principals, list):
            raise TypeError("principals must be a list.")


@dataclass(frozen=True)
class PolicyConfig(ModelBase):
    """Policy configuration with role bindings."""

    __doc_yaml_example__ = """\
bindings:
  - role_name: collaborator
    principals:
      - ug_abc123
  - role_name: readonly
    principals:
      - ug_def456
      - ug_ghi789
"""

    __doc_py_example__ = """\
from anyscale.policy.models import PolicyBinding, PolicyConfig

config = PolicyConfig(
    bindings=[
        PolicyBinding(role_name="collaborator", principals=["ug_abc123"]),
        PolicyBinding(role_name="readonly", principals=["ug_def456", "ug_ghi789"]),
    ]
)
"""

    bindings: List[PolicyBinding] = field(
        metadata={"docstring": "List of role bindings for the policy."}
    )

    def _validate_bindings(self, bindings: List[PolicyBinding]):
        if not isinstance(bindings, list):
            raise TypeError("bindings must be a list.")


@dataclass(frozen=True)
class ResourcePolicy(ModelBase):
    """Resource policy model representing permissions for a resource."""

    __doc_py_example__ = """\
import anyscale
from anyscale.policy.models import ResourcePolicy

policies = anyscale.policy.list(resource_type="cloud")
for policy in policies:
    print(f"{policy.resource_id}: {policy.bindings} (sync_status: {policy.sync_status})")
"""

    resource_id: str = field(metadata={"docstring": "The ID of the resource."})

    def _validate_resource_id(self, resource_id: str):
        if not isinstance(resource_id, str):
            raise TypeError("resource_id must be a string.")

    resource_type: str = field(
        metadata={"docstring": "The type of the resource (e.g., 'cloud', 'project')."}
    )

    def _validate_resource_type(self, resource_type: str):
        if not isinstance(resource_type, str):
            raise TypeError("resource_type must be a string.")

    bindings: List[PolicyBinding] = field(
        metadata={"docstring": "List of role bindings for the policy."}
    )

    def _validate_bindings(self, bindings: List[PolicyBinding]):
        if not isinstance(bindings, list):
            raise TypeError("bindings must be a list.")

    sync_status: PolicySyncStatus = field(
        metadata={
            "docstring": "Sync status of the policy (pending, success, or failed)."
        }
    )

    def _validate_sync_status(self, sync_status: PolicySyncStatus):
        if not isinstance(sync_status, PolicySyncStatus):
            raise TypeError("sync_status must be a PolicySyncStatus.")


@dataclass(frozen=True)
class Policy(ModelBase):
    """Policy model representing the policy for a single resource."""

    __doc_py_example__ = """\
import anyscale
from anyscale.policy.models import Policy

policy = anyscale.policy.get(resource_type="cloud", resource_id="cld_abc123")
print(f"Sync status: {policy.sync_status}")
for binding in policy.bindings:
    print(f"{binding.role_name}: {binding.principals}")
"""

    bindings: List[PolicyBinding] = field(
        metadata={"docstring": "List of role bindings for the policy."}
    )

    def _validate_bindings(self, bindings: List[PolicyBinding]):
        if not isinstance(bindings, list):
            raise TypeError("bindings must be a list.")

    sync_status: PolicySyncStatus = field(
        metadata={
            "docstring": "Sync status of the policy (pending, success, or failed)."
        }
    )

    def _validate_sync_status(self, sync_status: PolicySyncStatus):
        if not isinstance(sync_status, PolicySyncStatus):
            raise TypeError("sync_status must be a PolicySyncStatus.")



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OrganizationContractInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization_id': 'str',
        'organization_name': 'str',
        'metronome_customer_id': 'str',
        'customer_type': 'str',
        'contracts': 'list[ContractInfo]',
        'current_contract_type': 'str',
        'poc_end_date': 'datetime'
    }

    attribute_map = {
        'organization_id': 'organization_id',
        'organization_name': 'organization_name',
        'metronome_customer_id': 'metronome_customer_id',
        'customer_type': 'customer_type',
        'contracts': 'contracts',
        'current_contract_type': 'current_contract_type',
        'poc_end_date': 'poc_end_date'
    }

    def __init__(self, organization_id=None, organization_name=None, metronome_customer_id=None, customer_type=None, contracts=None, current_contract_type=None, poc_end_date=None, local_vars_configuration=None):  # noqa: E501
        """OrganizationContractInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._organization_id = None
        self._organization_name = None
        self._metronome_customer_id = None
        self._customer_type = None
        self._contracts = None
        self._current_contract_type = None
        self._poc_end_date = None
        self.discriminator = None

        self.organization_id = organization_id
        self.organization_name = organization_name
        self.metronome_customer_id = metronome_customer_id
        if customer_type is not None:
            self.customer_type = customer_type
        if contracts is not None:
            self.contracts = contracts
        if current_contract_type is not None:
            self.current_contract_type = current_contract_type
        if poc_end_date is not None:
            self.poc_end_date = poc_end_date

    @property
    def organization_id(self):
        """Gets the organization_id of this OrganizationContractInfo.  # noqa: E501

        ID of the organization  # noqa: E501

        :return: The organization_id of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrganizationContractInfo.

        ID of the organization  # noqa: E501

        :param organization_id: The organization_id of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def organization_name(self):
        """Gets the organization_name of this OrganizationContractInfo.  # noqa: E501

        Name of the organization  # noqa: E501

        :return: The organization_name of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._organization_name

    @organization_name.setter
    def organization_name(self, organization_name):
        """Sets the organization_name of this OrganizationContractInfo.

        Name of the organization  # noqa: E501

        :param organization_name: The organization_name of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_name is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_name`, must not be `None`")  # noqa: E501

        self._organization_name = organization_name

    @property
    def metronome_customer_id(self):
        """Gets the metronome_customer_id of this OrganizationContractInfo.  # noqa: E501

        Metronome customer ID  # noqa: E501

        :return: The metronome_customer_id of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._metronome_customer_id

    @metronome_customer_id.setter
    def metronome_customer_id(self, metronome_customer_id):
        """Sets the metronome_customer_id of this OrganizationContractInfo.

        Metronome customer ID  # noqa: E501

        :param metronome_customer_id: The metronome_customer_id of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and metronome_customer_id is None:  # noqa: E501
            raise ValueError("Invalid value for `metronome_customer_id`, must not be `None`")  # noqa: E501

        self._metronome_customer_id = metronome_customer_id

    @property
    def customer_type(self):
        """Gets the customer_type of this OrganizationContractInfo.  # noqa: E501

        Customer type  # noqa: E501

        :return: The customer_type of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._customer_type

    @customer_type.setter
    def customer_type(self, customer_type):
        """Sets the customer_type of this OrganizationContractInfo.

        Customer type  # noqa: E501

        :param customer_type: The customer_type of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """

        self._customer_type = customer_type

    @property
    def contracts(self):
        """Gets the contracts of this OrganizationContractInfo.  # noqa: E501

        List of contracts  # noqa: E501

        :return: The contracts of this OrganizationContractInfo.  # noqa: E501
        :rtype: list[ContractInfo]
        """
        return self._contracts

    @contracts.setter
    def contracts(self, contracts):
        """Sets the contracts of this OrganizationContractInfo.

        List of contracts  # noqa: E501

        :param contracts: The contracts of this OrganizationContractInfo.  # noqa: E501
        :type: list[ContractInfo]
        """

        self._contracts = contracts

    @property
    def current_contract_type(self):
        """Gets the current_contract_type of this OrganizationContractInfo.  # noqa: E501

        Type of current active contract  # noqa: E501

        :return: The current_contract_type of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._current_contract_type

    @current_contract_type.setter
    def current_contract_type(self, current_contract_type):
        """Sets the current_contract_type of this OrganizationContractInfo.

        Type of current active contract  # noqa: E501

        :param current_contract_type: The current_contract_type of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """

        self._current_contract_type = current_contract_type

    @property
    def poc_end_date(self):
        """Gets the poc_end_date of this OrganizationContractInfo.  # noqa: E501

        POC end date if in POC  # noqa: E501

        :return: The poc_end_date of this OrganizationContractInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._poc_end_date

    @poc_end_date.setter
    def poc_end_date(self, poc_end_date):
        """Sets the poc_end_date of this OrganizationContractInfo.

        POC end date if in POC  # noqa: E501

        :param poc_end_date: The poc_end_date of this OrganizationContractInfo.  # noqa: E501
        :type: datetime
        """

        self._poc_end_date = poc_end_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrganizationContractInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrganizationContractInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class KubernetesConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'anyscale_operator_iam_identity': 'str',
        'zones': 'list[str]'
    }

    attribute_map = {
        'anyscale_operator_iam_identity': 'anyscale_operator_iam_identity',
        'zones': 'zones'
    }

    def __init__(self, anyscale_operator_iam_identity=None, zones=None, local_vars_configuration=None):  # noqa: E501
        """KubernetesConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._anyscale_operator_iam_identity = None
        self._zones = None
        self.discriminator = None

        if anyscale_operator_iam_identity is not None:
            self.anyscale_operator_iam_identity = anyscale_operator_iam_identity
        if zones is not None:
            self.zones = zones

    @property
    def anyscale_operator_iam_identity(self):
        """Gets the anyscale_operator_iam_identity of this KubernetesConfig.  # noqa: E501

        The cloud provider IAM identity federated with the Anyscale Operator's Kubernetes service account, which will be used by Anyscale control plane for validation during Anyscale Operator bootstrap in the dataplane. IN AWS EKS, this is the ARN of the IAM role. For GCP GKE, this is the service account email.  # noqa: E501

        :return: The anyscale_operator_iam_identity of this KubernetesConfig.  # noqa: E501
        :rtype: str
        """
        return self._anyscale_operator_iam_identity

    @anyscale_operator_iam_identity.setter
    def anyscale_operator_iam_identity(self, anyscale_operator_iam_identity):
        """Sets the anyscale_operator_iam_identity of this KubernetesConfig.

        The cloud provider IAM identity federated with the Anyscale Operator's Kubernetes service account, which will be used by Anyscale control plane for validation during Anyscale Operator bootstrap in the dataplane. IN AWS EKS, this is the ARN of the IAM role. For GCP GKE, this is the service account email.  # noqa: E501

        :param anyscale_operator_iam_identity: The anyscale_operator_iam_identity of this KubernetesConfig.  # noqa: E501
        :type: str
        """

        self._anyscale_operator_iam_identity = anyscale_operator_iam_identity

    @property
    def zones(self):
        """Gets the zones of this KubernetesConfig.  # noqa: E501

        List of zones to launch pods in.  # noqa: E501

        :return: The zones of this KubernetesConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._zones

    @zones.setter
    def zones(self, zones):
        """Sets the zones of this KubernetesConfig.

        List of zones to launch pods in.  # noqa: E501

        :param zones: The zones of this KubernetesConfig.  # noqa: E501
        :type: list[str]
        """

        self._zones = zones

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KubernetesConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, KubernetesConfig):
            return True

        return self.to_dict() != other.to_dict()



import os
import re

from setuptools import find_packages, setup


# mypy: ignore-errors


def find_version(path):
    with open(path) as f:
        match = re.search(
            r"^__version__ = ['\"]([^'\"]*)['\"]", f.read(), re.MULTILINE,
        )
        if match:
            return match.group(1)
        raise RuntimeError("Unable to find version string.")


def read_requirements(path):
    with open(path) as f:
        return [line.strip() for line in f.readlines()]


_gcp_extra_requires = [
    "google-api-python-client",
    "google-cloud-secret-manager",
    "google-cloud-compute",
    "google-cloud-resource-manager",
    "google-cloud-filestore",
    "google-cloud-storage",
    "google-cloud-redis",
    "google-cloud-certificate-manager",
]

_all_extra_requires = _gcp_extra_requires + ["ray>=2.0.0"]

# If adding new webterminal deps,
# Update backend/server/services/application_templates_service.py
# to prevent users from uninstalling them.
_backend_extra_requires = _gcp_extra_requires + [
    "terminado",
    "tornado",
]

extras_require = {
    "gcp": _gcp_extra_requires,
    "all": _all_extra_requires,
    "backend": _backend_extra_requires,
}


def package_files(directory):
    paths = []
    for path, _, filenames in os.walk(directory):
        if os.path.basename(path) == "tests":  # skip test directory
            continue
        for filename in filenames:
            paths.append(os.path.join("..", path, filename))
    return paths


VERSION_PATH = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "anyscale", "version.py"
)
REQUIREMENTS_PATH = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "requirements.in"
)

with open("README.md") as fh:
    long_description = fh.read()


setup(
    name="anyscale",
    version=find_version(VERSION_PATH),
    author="Anyscale Inc.",
    description=("Command Line Interface for Anyscale"),
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=[*find_packages(exclude=["tests", "tests.*"])],
    setup_requires=["setuptools_scm"],
    python_requires=">=3.9,<3.14",
    package_data={
        "": [
            "anyscale/webterminal/bash-preexec.sh",
            "anyscale/_private/docgen/api.md",
            "anyscale/_private/docgen/models.md",
            "requirements.in",
            "*.yaml",
            "*.json",
            *package_files("anyscale/client"),
            *package_files("anyscale/sdk"),
        ],
    },
    install_requires=read_requirements(REQUIREMENTS_PATH),
    extras_require=extras_require,
    entry_points={"console_scripts": ["anyscale=anyscale.scripts:main"]},
    include_package_data=True,
    zip_safe=False,
    license_files=["LICENSE", "NOTICE"],
    license="AS License",
)



import json
import pickle
from enum import Enum
from pathlib import Path
from typing import Any, Callable, Union

from .types import StrBytes


class Protocol(str, Enum):
    json = 'json'
    pickle = 'pickle'


def load_str_bytes(
    b: StrBytes,
    *,
    content_type: str = None,
    encoding: str = 'utf8',
    proto: Protocol = None,
    allow_pickle: bool = False,
    json_loads: Callable[[str], Any] = json.loads,
) -> Any:
    if proto is None and content_type:
        if content_type.endswith(('json', 'javascript')):
            pass
        elif allow_pickle and content_type.endswith('pickle'):
            proto = Protocol.pickle
        else:
            raise TypeError(f'Unknown content-type: {content_type}')

    proto = proto or Protocol.json

    if proto == Protocol.json:
        if isinstance(b, bytes):
            b = b.decode(encoding)
        return json_loads(b)
    elif proto == Protocol.pickle:
        if not allow_pickle:
            raise RuntimeError('Trying to decode with pickle with allow_pickle=False')
        bb = b if isinstance(b, bytes) else b.encode()
        return pickle.loads(bb)
    else:
        raise TypeError(f'Unknown protocol: {proto}')


def load_file(
    path: Union[str, Path],
    *,
    content_type: str = None,
    encoding: str = 'utf8',
    proto: Protocol = None,
    allow_pickle: bool = False,
    json_loads: Callable[[str], Any] = json.loads,
) -> Any:
    path = Path(path)
    b = path.read_bytes()
    if content_type is None:
        if path.suffix in ('.js', '.json'):
            proto = Protocol.json
        elif path.suffix == '.pkl':
            proto = Protocol.pickle

    return load_str_bytes(
        b, proto=proto, content_type=content_type, encoding=encoding, allow_pickle=allow_pickle, json_loads=json_loads
    )



import re

from anyscale.sdk.anyscale_client.models.cluster_environment import ClusterEnvironment
from anyscale.sdk.anyscale_client.models.cluster_environment_build import (
    ClusterEnvironmentBuild,
)


DEFAULT_APP_CONFIG_ID_PREFIX = "DEFAULT_APP_CONFIG_ID"
IMAGE_URI_PATTERN = "[registry_host/]user_name/repository[:tag][@digest]"
IMAGE_URI_PATTERN_RE = re.compile(
    r"^"
    # Optional registry host: hostname with optional port
    r"((?P<host>[a-zA-Z0-9.-]+)(?::(?P<port>[0-9]+))?/)?"
    # Repository: user_name/repository
    r"(?P<user>[a-zA-Z0-9-_]+/)?"  # user is optional.
    r"(?P<repository>[a-zA-Z0-9-/_.]+)"
    # Optional Tag: version or string after ':'
    r"(:(?P<tag>[a-zA-Z0-9_.-]+))?"
    # Optional Digest: string after '@'
    # Note that when both tag and digest are provided, the tag is ignored.
    r"(@(?P<digest>[a-zA-Z0-9:]+))?"
    r"$"
)

ANYSCALE_CLUSTER_ENV_PREFIX = "anyscale/image/"
DEFAULT_IMAGE_URI_PREFIX = "anyscale/ray"


class ImageURI:
    image_uri: str

    @classmethod
    def from_str(cls, image_uri_str: str):
        if not isinstance(image_uri_str, str):
            raise TypeError("'image_uri' must be a string.")

        if image_uri_str:
            matches = IMAGE_URI_PATTERN_RE.match(image_uri_str)
            if not matches:
                raise ValueError(
                    f"Invalid image URI: '{image_uri_str}'. Must be in the format: '{IMAGE_URI_PATTERN}'."
                )
        instance = super().__new__(cls)
        instance.image_uri = image_uri_str
        return instance

    @classmethod
    def _is_byod_build(cls, build: ClusterEnvironmentBuild) -> bool:
        return build.is_byod or (not build.config_json and not build.containerfile)

    @classmethod
    def _is_default_build(cls, build: ClusterEnvironmentBuild) -> bool:
        return bool(
            isinstance(build.cluster_environment_id, str)
            and build.cluster_environment_id.startswith(DEFAULT_APP_CONFIG_ID_PREFIX)
        )

    @classmethod
    def from_cluster_env_build(
        cls,
        cluster_env: ClusterEnvironment,
        build: ClusterEnvironmentBuild,
        use_image_alias: bool = False,
    ):
        if cls._is_byod_build(build) and not use_image_alias:
            if build.docker_image_name:
                return ImageURI.from_str(build.docker_image_name)
            else:
                return ImageURI.from_str("")
        elif cls._is_default_build(build):
            return ImageURI.from_str(build.docker_image_name)
        else:
            image_uri_str = (
                ANYSCALE_CLUSTER_ENV_PREFIX + f"{cluster_env.name}:{build.revision}"
            )
            return ImageURI.from_str(image_uri_str)

    def to_cluster_env_name(self) -> str:
        """Convert the image URI to a cluster environment name."""
        pattern = re.compile("^[A-Za-z0-9_-]+$")
        # Keep only characters that match the pattern
        escaped = []
        for c in self.image_uri:
            if not pattern.match(c):
                escaped.append("-")
            else:
                escaped.append(c)
        return "".join(escaped)

    def is_cluster_env_image(self) -> bool:
        """Check if the image URI is a cluster environment image."""
        return self.image_uri.startswith(ANYSCALE_CLUSTER_ENV_PREFIX)

    def is_default_image(self) -> bool:
        """Check if the image URI is a default image."""
        return self.image_uri.startswith(DEFAULT_IMAGE_URI_PREFIX)

    def to_cluster_env_identifier(self) -> str:
        assert self.is_cluster_env_image()
        identifier = self.image_uri.split("/")[2]
        return identifier

    def __str__(self):
        return self.image_uri

    def __repr__(self):
        return self.image_uri

    def __eq__(self, other):
        if isinstance(other, ImageURI):
            return self.image_uri == other.image_uri
        return False

    def __hash__(self):
        return hash(self.image_uri)



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class LineageGraphNode(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'namespace_name': 'str',
        'uri': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'created_by': 'str',
        'updated_by': 'str',
        'workload_type': 'LineageWorkloadType',
        'anyscale_workload_id': 'str',
        'artifact_type': 'LineageArtifactType',
        'artifact_file_format': 'str',
        'node_type': 'LineageNodeType'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'namespace_name': 'namespace_name',
        'uri': 'uri',
        'created_at': 'created_at',
        'updated_at': 'updated_at',
        'created_by': 'created_by',
        'updated_by': 'updated_by',
        'workload_type': 'workload_type',
        'anyscale_workload_id': 'anyscale_workload_id',
        'artifact_type': 'artifact_type',
        'artifact_file_format': 'artifact_file_format',
        'node_type': 'node_type'
    }

    def __init__(self, id=None, name=None, namespace_name=None, uri=None, created_at=None, updated_at=None, created_by=None, updated_by=None, workload_type=None, anyscale_workload_id=None, artifact_type=None, artifact_file_format=None, node_type=None, local_vars_configuration=None):  # noqa: E501
        """LineageGraphNode - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._namespace_name = None
        self._uri = None
        self._created_at = None
        self._updated_at = None
        self._created_by = None
        self._updated_by = None
        self._workload_type = None
        self._anyscale_workload_id = None
        self._artifact_type = None
        self._artifact_file_format = None
        self._node_type = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.namespace_name = namespace_name
        self.uri = uri
        self.created_at = created_at
        self.updated_at = updated_at
        self.created_by = created_by
        self.updated_by = updated_by
        if workload_type is not None:
            self.workload_type = workload_type
        if anyscale_workload_id is not None:
            self.anyscale_workload_id = anyscale_workload_id
        if artifact_type is not None:
            self.artifact_type = artifact_type
        if artifact_file_format is not None:
            self.artifact_file_format = artifact_file_format
        self.node_type = node_type

    @property
    def id(self):
        """Gets the id of this LineageGraphNode.  # noqa: E501

        ID of the lineage node  # noqa: E501

        :return: The id of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this LineageGraphNode.

        ID of the lineage node  # noqa: E501

        :param id: The id of this LineageGraphNode.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this LineageGraphNode.  # noqa: E501

        Name of the node  # noqa: E501

        :return: The name of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this LineageGraphNode.

        Name of the node  # noqa: E501

        :param name: The name of this LineageGraphNode.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def namespace_name(self):
        """Gets the namespace_name of this LineageGraphNode.  # noqa: E501

        Namespace name  # noqa: E501

        :return: The namespace_name of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._namespace_name

    @namespace_name.setter
    def namespace_name(self, namespace_name):
        """Sets the namespace_name of this LineageGraphNode.

        Namespace name  # noqa: E501

        :param namespace_name: The namespace_name of this LineageGraphNode.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and namespace_name is None:  # noqa: E501
            raise ValueError("Invalid value for `namespace_name`, must not be `None`")  # noqa: E501

        self._namespace_name = namespace_name

    @property
    def uri(self):
        """Gets the uri of this LineageGraphNode.  # noqa: E501

        URI of the node  # noqa: E501

        :return: The uri of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._uri

    @uri.setter
    def uri(self, uri):
        """Sets the uri of this LineageGraphNode.

        URI of the node  # noqa: E501

        :param uri: The uri of this LineageGraphNode.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and uri is None:  # noqa: E501
            raise ValueError("Invalid value for `uri`, must not be `None`")  # noqa: E501

        self._uri = uri

    @property
    def created_at(self):
        """Gets the created_at of this LineageGraphNode.  # noqa: E501

        Creation datetime  # noqa: E501

        :return: The created_at of this LineageGraphNode.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this LineageGraphNode.

        Creation datetime  # noqa: E501

        :param created_at: The created_at of this LineageGraphNode.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this LineageGraphNode.  # noqa: E501

        Last update datetime  # noqa: E501

        :return: The updated_at of this LineageGraphNode.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this LineageGraphNode.

        Last update datetime  # noqa: E501

        :param updated_at: The updated_at of this LineageGraphNode.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_at is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_at`, must not be `None`")  # noqa: E501

        self._updated_at = updated_at

    @property
    def created_by(self):
        """Gets the created_by of this LineageGraphNode.  # noqa: E501

        User who created the node  # noqa: E501

        :return: The created_by of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this LineageGraphNode.

        User who created the node  # noqa: E501

        :param created_by: The created_by of this LineageGraphNode.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and created_by is None:  # noqa: E501
            raise ValueError("Invalid value for `created_by`, must not be `None`")  # noqa: E501

        self._created_by = created_by

    @property
    def updated_by(self):
        """Gets the updated_by of this LineageGraphNode.  # noqa: E501

        User who last updated the node  # noqa: E501

        :return: The updated_by of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._updated_by

    @updated_by.setter
    def updated_by(self, updated_by):
        """Sets the updated_by of this LineageGraphNode.

        User who last updated the node  # noqa: E501

        :param updated_by: The updated_by of this LineageGraphNode.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and updated_by is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_by`, must not be `None`")  # noqa: E501

        self._updated_by = updated_by

    @property
    def workload_type(self):
        """Gets the workload_type of this LineageGraphNode.  # noqa: E501

        Type of workload (WORKSPACE/SERVICE/JOB)  # noqa: E501

        :return: The workload_type of this LineageGraphNode.  # noqa: E501
        :rtype: LineageWorkloadType
        """
        return self._workload_type

    @workload_type.setter
    def workload_type(self, workload_type):
        """Sets the workload_type of this LineageGraphNode.

        Type of workload (WORKSPACE/SERVICE/JOB)  # noqa: E501

        :param workload_type: The workload_type of this LineageGraphNode.  # noqa: E501
        :type: LineageWorkloadType
        """

        self._workload_type = workload_type

    @property
    def anyscale_workload_id(self):
        """Gets the anyscale_workload_id of this LineageGraphNode.  # noqa: E501

        Anyscale workload ID  # noqa: E501

        :return: The anyscale_workload_id of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._anyscale_workload_id

    @anyscale_workload_id.setter
    def anyscale_workload_id(self, anyscale_workload_id):
        """Sets the anyscale_workload_id of this LineageGraphNode.

        Anyscale workload ID  # noqa: E501

        :param anyscale_workload_id: The anyscale_workload_id of this LineageGraphNode.  # noqa: E501
        :type: str
        """

        self._anyscale_workload_id = anyscale_workload_id

    @property
    def artifact_type(self):
        """Gets the artifact_type of this LineageGraphNode.  # noqa: E501

        Type of artifact (DATASET/MODEL)  # noqa: E501

        :return: The artifact_type of this LineageGraphNode.  # noqa: E501
        :rtype: LineageArtifactType
        """
        return self._artifact_type

    @artifact_type.setter
    def artifact_type(self, artifact_type):
        """Sets the artifact_type of this LineageGraphNode.

        Type of artifact (DATASET/MODEL)  # noqa: E501

        :param artifact_type: The artifact_type of this LineageGraphNode.  # noqa: E501
        :type: LineageArtifactType
        """

        self._artifact_type = artifact_type

    @property
    def artifact_file_format(self):
        """Gets the artifact_file_format of this LineageGraphNode.  # noqa: E501

        File format (e.g., parquet, csv)  # noqa: E501

        :return: The artifact_file_format of this LineageGraphNode.  # noqa: E501
        :rtype: str
        """
        return self._artifact_file_format

    @artifact_file_format.setter
    def artifact_file_format(self, artifact_file_format):
        """Sets the artifact_file_format of this LineageGraphNode.

        File format (e.g., parquet, csv)  # noqa: E501

        :param artifact_file_format: The artifact_file_format of this LineageGraphNode.  # noqa: E501
        :type: str
        """

        self._artifact_file_format = artifact_file_format

    @property
    def node_type(self):
        """Gets the node_type of this LineageGraphNode.  # noqa: E501

        Type of node (ARTIFACT/WORKLOAD)  # noqa: E501

        :return: The node_type of this LineageGraphNode.  # noqa: E501
        :rtype: LineageNodeType
        """
        return self._node_type

    @node_type.setter
    def node_type(self, node_type):
        """Sets the node_type of this LineageGraphNode.

        Type of node (ARTIFACT/WORKLOAD)  # noqa: E501

        :param node_type: The node_type of this LineageGraphNode.  # noqa: E501
        :type: LineageNodeType
        """
        if self.local_vars_configuration.client_side_validation and node_type is None:  # noqa: E501
            raise ValueError("Invalid value for `node_type`, must not be `None`")  # noqa: E501

        self._node_type = node_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LineageGraphNode):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, LineageGraphNode):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ConvertToSelfServiceRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization_id': 'str',
        'self_service_start_date': 'datetime',
        'end_current_contract': 'bool'
    }

    attribute_map = {
        'organization_id': 'organization_id',
        'self_service_start_date': 'self_service_start_date',
        'end_current_contract': 'end_current_contract'
    }

    def __init__(self, organization_id=None, self_service_start_date=None, end_current_contract=True, local_vars_configuration=None):  # noqa: E501
        """ConvertToSelfServiceRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._organization_id = None
        self._self_service_start_date = None
        self._end_current_contract = None
        self.discriminator = None

        self.organization_id = organization_id
        if self_service_start_date is not None:
            self.self_service_start_date = self_service_start_date
        if end_current_contract is not None:
            self.end_current_contract = end_current_contract

    @property
    def organization_id(self):
        """Gets the organization_id of this ConvertToSelfServiceRequest.  # noqa: E501

        ID of the organization  # noqa: E501

        :return: The organization_id of this ConvertToSelfServiceRequest.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this ConvertToSelfServiceRequest.

        ID of the organization  # noqa: E501

        :param organization_id: The organization_id of this ConvertToSelfServiceRequest.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def self_service_start_date(self):
        """Gets the self_service_start_date of this ConvertToSelfServiceRequest.  # noqa: E501

        When self-service should start (defaults to now)  # noqa: E501

        :return: The self_service_start_date of this ConvertToSelfServiceRequest.  # noqa: E501
        :rtype: datetime
        """
        return self._self_service_start_date

    @self_service_start_date.setter
    def self_service_start_date(self, self_service_start_date):
        """Sets the self_service_start_date of this ConvertToSelfServiceRequest.

        When self-service should start (defaults to now)  # noqa: E501

        :param self_service_start_date: The self_service_start_date of this ConvertToSelfServiceRequest.  # noqa: E501
        :type: datetime
        """

        self._self_service_start_date = self_service_start_date

    @property
    def end_current_contract(self):
        """Gets the end_current_contract of this ConvertToSelfServiceRequest.  # noqa: E501

        Whether to end current contract  # noqa: E501

        :return: The end_current_contract of this ConvertToSelfServiceRequest.  # noqa: E501
        :rtype: bool
        """
        return self._end_current_contract

    @end_current_contract.setter
    def end_current_contract(self, end_current_contract):
        """Sets the end_current_contract of this ConvertToSelfServiceRequest.

        Whether to end current contract  # noqa: E501

        :param end_current_contract: The end_current_contract of this ConvertToSelfServiceRequest.  # noqa: E501
        :type: bool
        """

        self._end_current_contract = end_current_contract

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConvertToSelfServiceRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ConvertToSelfServiceRequest):
            return True

        return self.to_dict() != other.to_dict()



import contextlib
from dataclasses import dataclass, field, fields
from datetime import datetime
import json
from typing import Any, ClassVar, Dict, List, Optional, Union

from anyscale._private.models import ModelBase, ModelEnum


class ProjectPermissionLevel(ModelEnum):
    """Permission levels for project collaborators."""

    OWNER = "OWNER"
    WRITE = "WRITE"
    READONLY = "READONLY"

    __docstrings__: ClassVar[Dict[str, str]] = {
        OWNER: "Owner permission level for the project",
        WRITE: "Write permission level for the project",
        READONLY: "Readonly permission level for the project",
    }


@dataclass(frozen=True)
class CreateProjectCollaborator(ModelBase):
    """User to be added as a collaborator to a project.
    """

    __doc_py_example__ = """\
import anyscale
from anyscale.project.models import ProjectPermissionLevel, CreateProjectCollaborator
create_project_collaborator = CreateProjectCollaborator(
   # Email of the user to be added as a collaborator
    email="test@anyscale.com",
    # Permission level for the user to the project (ProjectPermissionLevel.OWNER, ProjectPermissionLevel.WRITE, ProjectPermissionLevel.READONLY)
    permission_level=ProjectPermissionLevel.READONLY,
)
"""

    def _validate_email(self, email: str):
        if not isinstance(email, str):
            raise TypeError("Email must be a string.")

    email: str = field(
        metadata={"docstring": "Email of the user to be added as a collaborator."},
    )

    def _validate_permission_level(
        self, permission_level: ProjectPermissionLevel
    ) -> ProjectPermissionLevel:
        if isinstance(permission_level, str):
            return ProjectPermissionLevel.validate(permission_level)
        elif isinstance(permission_level, ProjectPermissionLevel):
            return permission_level
        else:
            raise TypeError(
                f"'permission_level' must be a 'ProjectPermissionLevel' (it is {type(permission_level)})."
            )

    permission_level: ProjectPermissionLevel = field(  # type: ignore
        default=ProjectPermissionLevel.READONLY,  # type: ignore
        metadata={
            "docstring": "Permission level the added user should have for the project"  # type: ignore
            f"(one of: {','.join([str(m.value) for m in ProjectPermissionLevel])}",  # type: ignore
        },
    )


@dataclass(frozen=True)
class CreateProjectCollaborators(ModelBase):
    """List of users to be added as collaborators to a project.
    """

    __doc_py_example__ = """\
import anyscale
from anyscale.project.models import ProjectPermissionLevel, CreateProjectCollaborator, CreateProjectCollaborators
create_project_collaborator = CreateProjectCollaborator(
   # Email of the user to be added as a collaborator
    email="test@anyscale.com",
    # Permission level for the user to the project (ProjectPermissionLevel.OWNER, ProjectPermissionLevel.WRITE, ProjectPermissionLevel.READONLY)
    permission_level=ProjectPermissionLevel.READONLY,
)
create_project_collaborators = CreateProjectCollaborators(
    collaborators=[create_project_collaborator]
)
"""

    collaborators: List[Dict[str, Any]] = field(
        metadata={
            "docstring": "List of users to be added as collaborators to a project."
        },
    )

    def _validate_collaborators(self, collaborators: List[Dict[str, Any]]):
        if not isinstance(collaborators, list):
            raise TypeError("Collaborators must be a list.")


@dataclass(frozen=True)
class ProjectMinimal(ModelBase):
    """Minimal Project object."""

    id: str = field(metadata={"docstring": "ID of the project."},)

    def _validate_id(self, id: str):  # noqa: A002
        if not isinstance(id, str):
            raise TypeError("'id' must be a string.")

    name: str = field(metadata={"docstring": "Name of the project."},)

    def _validate_name(self, name: str):
        if not isinstance(name, str):
            raise TypeError("'name' must be a string.")

    description: str = field(metadata={"docstring": "Description of the project."},)

    def _validate_description(self, description: str):
        if not isinstance(description, str):
            raise TypeError("'description' must be a string.")

    created_at: str = field(
        metadata={"docstring": "Datetime of the project creation."},
    )

    def _validate_created_at(self, created_at: str):
        if not isinstance(created_at, str):
            raise TypeError("'created_at' must be a string.")

    creator_id: Optional[str] = field(
        default=None, metadata={"docstring": "ID of the creator of the project."},
    )

    def _validate_creator_id(self, creator_id: Optional[str]):
        if creator_id is not None and not isinstance(creator_id, str):
            raise TypeError("'creator_id' must be a string.")

    parent_cloud_id: Optional[str] = field(
        default=None, metadata={"docstring": "ID of the parent cloud."},
    )

    def _validate_parent_cloud_id(self, parent_cloud_id: Optional[str]):
        if parent_cloud_id is not None and not isinstance(parent_cloud_id, str):
            raise TypeError("'parent_cloud_id' must be a string.")

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ProjectMinimal":
        # remove any fields that are not in the dataclass
        valid_fields = {field_.name for field_ in fields(cls)}
        filtered_data = {k: v for k, v in data.items() if k in valid_fields}
        return cls(**filtered_data)


@dataclass(frozen=True)
class Project(ProjectMinimal):
    """Project object.
    """

    __doc_py_example__ = """\
import anyscale
from anyscale.project.models import Project

project: Project = anyscale.project.get(project_id="my-project-id")
"""

    is_owner: bool = field(
        default=False,
        metadata={"docstring": "Whether the user is the owner of the project."},
    )

    def _validate_is_owner(self, is_owner: bool):
        if not isinstance(is_owner, bool):
            raise TypeError("'is_owner' must be a boolean.")

    is_read_only: bool = field(
        default=False,
        metadata={"docstring": "Whether the user has read-only access to the project."},
    )

    def _validate_is_read_only(self, is_read_only: bool):
        if not isinstance(is_read_only, bool):
            raise TypeError("'is_read_only' must be a boolean.")

    directory_name: str = field(
        default="",
        metadata={
            "docstring": "Directory name of project to be used as working directory of clusters."
        },
    )

    def _validate_directory_name(self, directory_name: str):
        if not isinstance(directory_name, str):
            raise TypeError("'directory_name' must be a string.")

    is_default: bool = field(
        default=False,
        metadata={
            "docstring": "Whether the project is the default project for the organization."
        },
    )

    def _validate_is_default(self, is_default: bool):
        if not isinstance(is_default, bool):
            raise TypeError("'is_default' must be a boolean.")

    initial_cluster_config: Optional[Union[str, Dict[str, Any]]] = field(
        default=None,
        metadata={"docstring": "Initial cluster config associated with the project."},
    )

    def _validate_initial_cluster_config(
        self, initial_cluster_config: Union[str, Dict[str, Any], None]
    ):
        if initial_cluster_config is not None and not isinstance(
            initial_cluster_config, (str, dict)
        ):
            raise TypeError("'initial_cluster_config' must be a string or dictionary.")

    last_used_cloud_id: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "ID of the last cloud used in this project, or by the user if this is a new project."
        },
    )

    def _validate_last_used_cloud_id(self, last_used_cloud_id: Optional[str]):
        if last_used_cloud_id is not None and not isinstance(last_used_cloud_id, str):
            raise TypeError("'last_used_cloud_id' must be a string.")

    owners: List[str] = field(
        default_factory=list,
        metadata={
            "docstring": "List of IDs of users who have owner access to the project."
        },
    )

    def _validate_owners(self, owners: List[str]):
        if not isinstance(owners, list):
            raise TypeError("'owners' must be a list.")
        for owner in owners:
            if not isinstance(owner, str):
                raise TypeError(f"'{owner}' must be a string.")

    @classmethod
    def from_dict(cls, d: Dict[str, Any]) -> "Project":
        data = d.copy()
        # convert datetime fields to string
        for field_ in fields(cls):
            if field_.name in data and isinstance(data[field_.name], datetime):
                data[field_.name] = data[field_.name].strftime("%Y-%m-%d %H:%M:%S")

        # convert initial_cluster_config JSON string to dict
        if "initial_cluster_config" in data and isinstance(
            data["initial_cluster_config"], str
        ):
            with contextlib.suppress(json.JSONDecodeError):
                data["initial_cluster_config"] = json.loads(
                    data["initial_cluster_config"]
                )

        # convert owners list of dicts to list of string IDs
        if "owners" in data and isinstance(data["owners"], list):
            data["owners"] = [owner.get("id") for owner in data["owners"]]

        # remove any fields that are not in the dataclass
        valid_fields = {field_.name for field_ in fields(cls)}
        filtered_data = {k: v for k, v in data.items() if k in valid_fields}
        return cls(**filtered_data)


class ProjectSortField(ModelEnum):
    """Field to sort projects by."""

    NAME = "NAME"

    __docstrings__: ClassVar[Dict[str, str]] = {
        NAME: "Sort by project name.",
    }


class ProjectSortOrder(ModelEnum):
    """Direction of sorting."""

    ASC = "ASC"
    DESC = "DESC"

    __docstrings__: ClassVar[Dict[str, str]] = {
        ASC: "Sort in ascending order.",
        DESC: "Sort in descending order.",
    }



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ApplyAutoscalingConfigUpdateModel(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'service_version_id': 'str',
        'application_name': 'str',
        'deployment_name': 'str',
        'max_ongoing_requests': 'int',
        'autoscaling_enabled': 'bool',
        'min_replicas': 'int',
        'max_replicas': 'int',
        'target_ongoing_requests': 'int',
        'num_replicas': 'object'
    }

    attribute_map = {
        'service_version_id': 'service_version_id',
        'application_name': 'application_name',
        'deployment_name': 'deployment_name',
        'max_ongoing_requests': 'max_ongoing_requests',
        'autoscaling_enabled': 'autoscaling_enabled',
        'min_replicas': 'min_replicas',
        'max_replicas': 'max_replicas',
        'target_ongoing_requests': 'target_ongoing_requests',
        'num_replicas': 'num_replicas'
    }

    def __init__(self, service_version_id=None, application_name=None, deployment_name=None, max_ongoing_requests=None, autoscaling_enabled=None, min_replicas=None, max_replicas=None, target_ongoing_requests=None, num_replicas=None, local_vars_configuration=None):  # noqa: E501
        """ApplyAutoscalingConfigUpdateModel - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._service_version_id = None
        self._application_name = None
        self._deployment_name = None
        self._max_ongoing_requests = None
        self._autoscaling_enabled = None
        self._min_replicas = None
        self._max_replicas = None
        self._target_ongoing_requests = None
        self._num_replicas = None
        self.discriminator = None

        self.service_version_id = service_version_id
        self.application_name = application_name
        self.deployment_name = deployment_name
        if max_ongoing_requests is not None:
            self.max_ongoing_requests = max_ongoing_requests
        self.autoscaling_enabled = autoscaling_enabled
        if min_replicas is not None:
            self.min_replicas = min_replicas
        if max_replicas is not None:
            self.max_replicas = max_replicas
        if target_ongoing_requests is not None:
            self.target_ongoing_requests = target_ongoing_requests
        if num_replicas is not None:
            self.num_replicas = num_replicas

    @property
    def service_version_id(self):
        """Gets the service_version_id of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Id of the Service Version  # noqa: E501

        :return: The service_version_id of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: str
        """
        return self._service_version_id

    @service_version_id.setter
    def service_version_id(self, service_version_id):
        """Sets the service_version_id of this ApplyAutoscalingConfigUpdateModel.

        Id of the Service Version  # noqa: E501

        :param service_version_id: The service_version_id of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and service_version_id is None:  # noqa: E501
            raise ValueError("Invalid value for `service_version_id`, must not be `None`")  # noqa: E501

        self._service_version_id = service_version_id

    @property
    def application_name(self):
        """Gets the application_name of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Name of the Application  # noqa: E501

        :return: The application_name of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: str
        """
        return self._application_name

    @application_name.setter
    def application_name(self, application_name):
        """Sets the application_name of this ApplyAutoscalingConfigUpdateModel.

        Name of the Application  # noqa: E501

        :param application_name: The application_name of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and application_name is None:  # noqa: E501
            raise ValueError("Invalid value for `application_name`, must not be `None`")  # noqa: E501

        self._application_name = application_name

    @property
    def deployment_name(self):
        """Gets the deployment_name of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Name of the Deployment  # noqa: E501

        :return: The deployment_name of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: str
        """
        return self._deployment_name

    @deployment_name.setter
    def deployment_name(self, deployment_name):
        """Sets the deployment_name of this ApplyAutoscalingConfigUpdateModel.

        Name of the Deployment  # noqa: E501

        :param deployment_name: The deployment_name of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and deployment_name is None:  # noqa: E501
            raise ValueError("Invalid value for `deployment_name`, must not be `None`")  # noqa: E501

        self._deployment_name = deployment_name

    @property
    def max_ongoing_requests(self):
        """Gets the max_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Maximum number of ongoing requests per deployment  # noqa: E501

        :return: The max_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: int
        """
        return self._max_ongoing_requests

    @max_ongoing_requests.setter
    def max_ongoing_requests(self, max_ongoing_requests):
        """Sets the max_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.

        Maximum number of ongoing requests per deployment  # noqa: E501

        :param max_ongoing_requests: The max_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: int
        """

        self._max_ongoing_requests = max_ongoing_requests

    @property
    def autoscaling_enabled(self):
        """Gets the autoscaling_enabled of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Whether the autoscaling is enabled  # noqa: E501

        :return: The autoscaling_enabled of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: bool
        """
        return self._autoscaling_enabled

    @autoscaling_enabled.setter
    def autoscaling_enabled(self, autoscaling_enabled):
        """Sets the autoscaling_enabled of this ApplyAutoscalingConfigUpdateModel.

        Whether the autoscaling is enabled  # noqa: E501

        :param autoscaling_enabled: The autoscaling_enabled of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and autoscaling_enabled is None:  # noqa: E501
            raise ValueError("Invalid value for `autoscaling_enabled`, must not be `None`")  # noqa: E501

        self._autoscaling_enabled = autoscaling_enabled

    @property
    def min_replicas(self):
        """Gets the min_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Minimum number of replicas  # noqa: E501

        :return: The min_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: int
        """
        return self._min_replicas

    @min_replicas.setter
    def min_replicas(self, min_replicas):
        """Sets the min_replicas of this ApplyAutoscalingConfigUpdateModel.

        Minimum number of replicas  # noqa: E501

        :param min_replicas: The min_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: int
        """

        self._min_replicas = min_replicas

    @property
    def max_replicas(self):
        """Gets the max_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Maximum number of replicas  # noqa: E501

        :return: The max_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: int
        """
        return self._max_replicas

    @max_replicas.setter
    def max_replicas(self, max_replicas):
        """Sets the max_replicas of this ApplyAutoscalingConfigUpdateModel.

        Maximum number of replicas  # noqa: E501

        :param max_replicas: The max_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: int
        """

        self._max_replicas = max_replicas

    @property
    def target_ongoing_requests(self):
        """Gets the target_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Target number of ongoing requests per deployment  # noqa: E501

        :return: The target_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: int
        """
        return self._target_ongoing_requests

    @target_ongoing_requests.setter
    def target_ongoing_requests(self, target_ongoing_requests):
        """Sets the target_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.

        Target number of ongoing requests per deployment  # noqa: E501

        :param target_ongoing_requests: The target_ongoing_requests of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: int
        """

        self._target_ongoing_requests = target_ongoing_requests

    @property
    def num_replicas(self):
        """Gets the num_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501

        Number of replicas  # noqa: E501

        :return: The num_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :rtype: object
        """
        return self._num_replicas

    @num_replicas.setter
    def num_replicas(self, num_replicas):
        """Sets the num_replicas of this ApplyAutoscalingConfigUpdateModel.

        Number of replicas  # noqa: E501

        :param num_replicas: The num_replicas of this ApplyAutoscalingConfigUpdateModel.  # noqa: E501
        :type: object
        """

        self._num_replicas = num_replicas

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ApplyAutoscalingConfigUpdateModel):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ApplyAutoscalingConfigUpdateModel):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class CloudDeployment(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cloud_resource_id': 'str',
        'cloud_deployment_id': 'str',
        'name': 'str',
        'provider': 'CloudProviders',
        'compute_stack': 'ComputeStack',
        'region': 'str',
        'networking_mode': 'NetworkingMode',
        'object_storage': 'ObjectStorage',
        'file_storage': 'FileStorage',
        'aws_config': 'AWSConfig',
        'gcp_config': 'GCPConfig',
        'azure_config': 'AzureConfig',
        'kubernetes_config': 'KubernetesConfig'
    }

    attribute_map = {
        'cloud_resource_id': 'cloud_resource_id',
        'cloud_deployment_id': 'cloud_deployment_id',
        'name': 'name',
        'provider': 'provider',
        'compute_stack': 'compute_stack',
        'region': 'region',
        'networking_mode': 'networking_mode',
        'object_storage': 'object_storage',
        'file_storage': 'file_storage',
        'aws_config': 'aws_config',
        'gcp_config': 'gcp_config',
        'azure_config': 'azure_config',
        'kubernetes_config': 'kubernetes_config'
    }

    def __init__(self, cloud_resource_id=None, cloud_deployment_id=None, name=None, provider=None, compute_stack=None, region=None, networking_mode=None, object_storage=None, file_storage=None, aws_config=None, gcp_config=None, azure_config=None, kubernetes_config=None, local_vars_configuration=None):  # noqa: E501
        """CloudDeployment - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._cloud_resource_id = None
        self._cloud_deployment_id = None
        self._name = None
        self._provider = None
        self._compute_stack = None
        self._region = None
        self._networking_mode = None
        self._object_storage = None
        self._file_storage = None
        self._aws_config = None
        self._gcp_config = None
        self._azure_config = None
        self._kubernetes_config = None
        self.discriminator = None

        if cloud_resource_id is not None:
            self.cloud_resource_id = cloud_resource_id
        if cloud_deployment_id is not None:
            self.cloud_deployment_id = cloud_deployment_id
        if name is not None:
            self.name = name
        self.provider = provider
        if compute_stack is not None:
            self.compute_stack = compute_stack
        if region is not None:
            self.region = region
        if networking_mode is not None:
            self.networking_mode = networking_mode
        self.object_storage = object_storage
        self.file_storage = file_storage
        self.aws_config = aws_config
        self.gcp_config = gcp_config
        self.azure_config = azure_config
        self.kubernetes_config = kubernetes_config

    @property
    def cloud_resource_id(self):
        """Gets the cloud_resource_id of this CloudDeployment.  # noqa: E501

        Unique identifier for this cloud resource.  # noqa: E501

        :return: The cloud_resource_id of this CloudDeployment.  # noqa: E501
        :rtype: str
        """
        return self._cloud_resource_id

    @cloud_resource_id.setter
    def cloud_resource_id(self, cloud_resource_id):
        """Sets the cloud_resource_id of this CloudDeployment.

        Unique identifier for this cloud resource.  # noqa: E501

        :param cloud_resource_id: The cloud_resource_id of this CloudDeployment.  # noqa: E501
        :type: str
        """

        self._cloud_resource_id = cloud_resource_id

    @property
    def cloud_deployment_id(self):
        """Gets the cloud_deployment_id of this CloudDeployment.  # noqa: E501

        DEPRECATED. Use cloud_resource_id instead.  # noqa: E501

        :return: The cloud_deployment_id of this CloudDeployment.  # noqa: E501
        :rtype: str
        """
        return self._cloud_deployment_id

    @cloud_deployment_id.setter
    def cloud_deployment_id(self, cloud_deployment_id):
        """Sets the cloud_deployment_id of this CloudDeployment.

        DEPRECATED. Use cloud_resource_id instead.  # noqa: E501

        :param cloud_deployment_id: The cloud_deployment_id of this CloudDeployment.  # noqa: E501
        :type: str
        """

        self._cloud_deployment_id = cloud_deployment_id

    @property
    def name(self):
        """Gets the name of this CloudDeployment.  # noqa: E501

        The name of this cloud resource.  # noqa: E501

        :return: The name of this CloudDeployment.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this CloudDeployment.

        The name of this cloud resource.  # noqa: E501

        :param name: The name of this CloudDeployment.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def provider(self):
        """Gets the provider of this CloudDeployment.  # noqa: E501

        The cloud provider type (AWS, GCP, AZURE, or GENERIC).  # noqa: E501

        :return: The provider of this CloudDeployment.  # noqa: E501
        :rtype: CloudProviders
        """
        return self._provider

    @provider.setter
    def provider(self, provider):
        """Sets the provider of this CloudDeployment.

        The cloud provider type (AWS, GCP, AZURE, or GENERIC).  # noqa: E501

        :param provider: The provider of this CloudDeployment.  # noqa: E501
        :type: CloudProviders
        """
        if self.local_vars_configuration.client_side_validation and provider is None:  # noqa: E501
            raise ValueError("Invalid value for `provider`, must not be `None`")  # noqa: E501

        self._provider = provider

    @property
    def compute_stack(self):
        """Gets the compute_stack of this CloudDeployment.  # noqa: E501

        The compute stack (VM or K8S).  # noqa: E501

        :return: The compute_stack of this CloudDeployment.  # noqa: E501
        :rtype: ComputeStack
        """
        return self._compute_stack

    @compute_stack.setter
    def compute_stack(self, compute_stack):
        """Sets the compute_stack of this CloudDeployment.

        The compute stack (VM or K8S).  # noqa: E501

        :param compute_stack: The compute_stack of this CloudDeployment.  # noqa: E501
        :type: ComputeStack
        """

        self._compute_stack = compute_stack

    @property
    def region(self):
        """Gets the region of this CloudDeployment.  # noqa: E501

        The region (e.g., us-west-2).  # noqa: E501

        :return: The region of this CloudDeployment.  # noqa: E501
        :rtype: str
        """
        return self._region

    @region.setter
    def region(self, region):
        """Sets the region of this CloudDeployment.

        The region (e.g., us-west-2).  # noqa: E501

        :param region: The region of this CloudDeployment.  # noqa: E501
        :type: str
        """

        self._region = region

    @property
    def networking_mode(self):
        """Gets the networking_mode of this CloudDeployment.  # noqa: E501

        Whether to use public or private networking.  # noqa: E501

        :return: The networking_mode of this CloudDeployment.  # noqa: E501
        :rtype: NetworkingMode
        """
        return self._networking_mode

    @networking_mode.setter
    def networking_mode(self, networking_mode):
        """Sets the networking_mode of this CloudDeployment.

        Whether to use public or private networking.  # noqa: E501

        :param networking_mode: The networking_mode of this CloudDeployment.  # noqa: E501
        :type: NetworkingMode
        """

        self._networking_mode = networking_mode

    @property
    def object_storage(self):
        """Gets the object_storage of this CloudDeployment.  # noqa: E501

        Object storage configuration.  # noqa: E501

        :return: The object_storage of this CloudDeployment.  # noqa: E501
        :rtype: ObjectStorage
        """
        return self._object_storage

    @object_storage.setter
    def object_storage(self, object_storage):
        """Sets the object_storage of this CloudDeployment.

        Object storage configuration.  # noqa: E501

        :param object_storage: The object_storage of this CloudDeployment.  # noqa: E501
        :type: ObjectStorage
        """

        self._object_storage = object_storage

    @property
    def file_storage(self):
        """Gets the file_storage of this CloudDeployment.  # noqa: E501

        File storage configuration.  # noqa: E501

        :return: The file_storage of this CloudDeployment.  # noqa: E501
        :rtype: FileStorage
        """
        return self._file_storage

    @file_storage.setter
    def file_storage(self, file_storage):
        """Sets the file_storage of this CloudDeployment.

        File storage configuration.  # noqa: E501

        :param file_storage: The file_storage of this CloudDeployment.  # noqa: E501
        :type: FileStorage
        """

        self._file_storage = file_storage

    @property
    def aws_config(self):
        """Gets the aws_config of this CloudDeployment.  # noqa: E501

        AWS provider-specific configurations.  # noqa: E501

        :return: The aws_config of this CloudDeployment.  # noqa: E501
        :rtype: AWSConfig
        """
        return self._aws_config

    @aws_config.setter
    def aws_config(self, aws_config):
        """Sets the aws_config of this CloudDeployment.

        AWS provider-specific configurations.  # noqa: E501

        :param aws_config: The aws_config of this CloudDeployment.  # noqa: E501
        :type: AWSConfig
        """

        self._aws_config = aws_config

    @property
    def gcp_config(self):
        """Gets the gcp_config of this CloudDeployment.  # noqa: E501

        GCP provider-specific configurations.  # noqa: E501

        :return: The gcp_config of this CloudDeployment.  # noqa: E501
        :rtype: GCPConfig
        """
        return self._gcp_config

    @gcp_config.setter
    def gcp_config(self, gcp_config):
        """Sets the gcp_config of this CloudDeployment.

        GCP provider-specific configurations.  # noqa: E501

        :param gcp_config: The gcp_config of this CloudDeployment.  # noqa: E501
        :type: GCPConfig
        """

        self._gcp_config = gcp_config

    @property
    def azure_config(self):
        """Gets the azure_config of this CloudDeployment.  # noqa: E501

        Azure provider-specific configurations.  # noqa: E501

        :return: The azure_config of this CloudDeployment.  # noqa: E501
        :rtype: AzureConfig
        """
        return self._azure_config

    @azure_config.setter
    def azure_config(self, azure_config):
        """Sets the azure_config of this CloudDeployment.

        Azure provider-specific configurations.  # noqa: E501

        :param azure_config: The azure_config of this CloudDeployment.  # noqa: E501
        :type: AzureConfig
        """

        self._azure_config = azure_config

    @property
    def kubernetes_config(self):
        """Gets the kubernetes_config of this CloudDeployment.  # noqa: E501

        Kubernetes stack configurations.  # noqa: E501

        :return: The kubernetes_config of this CloudDeployment.  # noqa: E501
        :rtype: KubernetesConfig
        """
        return self._kubernetes_config

    @kubernetes_config.setter
    def kubernetes_config(self, kubernetes_config):
        """Sets the kubernetes_config of this CloudDeployment.

        Kubernetes stack configurations.  # noqa: E501

        :param kubernetes_config: The kubernetes_config of this CloudDeployment.  # noqa: E501
        :type: KubernetesConfig
        """

        self._kubernetes_config = kubernetes_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CloudDeployment):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CloudDeployment):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ApplyProductionServiceMultiVersionV2Model(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'rollout_strategy': 'RolloutStrategy',
        'cloud_id': 'str',
        'project_id': 'str',
        'service_versions': 'list[ApplyProductionServiceV2Model]'
    }

    attribute_map = {
        'rollout_strategy': 'rollout_strategy',
        'cloud_id': 'cloud_id',
        'project_id': 'project_id',
        'service_versions': 'service_versions'
    }

    def __init__(self, rollout_strategy=None, cloud_id=None, project_id=None, service_versions=None, local_vars_configuration=None):  # noqa: E501
        """ApplyProductionServiceMultiVersionV2Model - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._rollout_strategy = None
        self._cloud_id = None
        self._project_id = None
        self._service_versions = None
        self.discriminator = None

        if rollout_strategy is not None:
            self.rollout_strategy = rollout_strategy
        if cloud_id is not None:
            self.cloud_id = cloud_id
        if project_id is not None:
            self.project_id = project_id
        self.service_versions = service_versions

    @property
    def rollout_strategy(self):
        """Gets the rollout_strategy of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501

        Strategy for rollout. The ROLLOUT strategy will deploy your Ray Serve configuration onto a newly started cluster, and then shift traffic over to the new cluster. You can manually control the speed of the rollout using the canary_percent configuration. The IN_PLACE strategy will use Ray Serve in place upgrade to update your existing cluster in place. When using this rollout strategy, you may only change the ray_serve_config field. You cannot partially shift traffic or rollback an in place upgrade. In place upgrades are faster and riskier than rollouts, and we recommend only using them for relatively safe changes (for example, increasing the number of replicas on a Ray Serve deployment). The MULTI_VERSION strategy will deploy multiple versions of the service. You can shift traffic between versions using the traffic_percent configuration.Default strategy is ROLLOUT.  # noqa: E501

        :return: The rollout_strategy of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :rtype: RolloutStrategy
        """
        return self._rollout_strategy

    @rollout_strategy.setter
    def rollout_strategy(self, rollout_strategy):
        """Sets the rollout_strategy of this ApplyProductionServiceMultiVersionV2Model.

        Strategy for rollout. The ROLLOUT strategy will deploy your Ray Serve configuration onto a newly started cluster, and then shift traffic over to the new cluster. You can manually control the speed of the rollout using the canary_percent configuration. The IN_PLACE strategy will use Ray Serve in place upgrade to update your existing cluster in place. When using this rollout strategy, you may only change the ray_serve_config field. You cannot partially shift traffic or rollback an in place upgrade. In place upgrades are faster and riskier than rollouts, and we recommend only using them for relatively safe changes (for example, increasing the number of replicas on a Ray Serve deployment). The MULTI_VERSION strategy will deploy multiple versions of the service. You can shift traffic between versions using the traffic_percent configuration.Default strategy is ROLLOUT.  # noqa: E501

        :param rollout_strategy: The rollout_strategy of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :type: RolloutStrategy
        """

        self._rollout_strategy = rollout_strategy

    @property
    def cloud_id(self):
        """Gets the cloud_id of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501

        Id of the cloud this Service will launch clusters in.  # noqa: E501

        :return: The cloud_id of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this ApplyProductionServiceMultiVersionV2Model.

        Id of the cloud this Service will launch clusters in.  # noqa: E501

        :param cloud_id: The cloud_id of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :type: str
        """

        self._cloud_id = cloud_id

    @property
    def project_id(self):
        """Gets the project_id of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501

        Id of the project this Service will start clusters in.  # noqa: E501

        :return: The project_id of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this ApplyProductionServiceMultiVersionV2Model.

        Id of the project this Service will start clusters in.  # noqa: E501

        :param project_id: The project_id of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def service_versions(self):
        """Gets the service_versions of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501

        List of versions to deploy. Each version is a deployment of the service.  # noqa: E501

        :return: The service_versions of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :rtype: list[ApplyProductionServiceV2Model]
        """
        return self._service_versions

    @service_versions.setter
    def service_versions(self, service_versions):
        """Sets the service_versions of this ApplyProductionServiceMultiVersionV2Model.

        List of versions to deploy. Each version is a deployment of the service.  # noqa: E501

        :param service_versions: The service_versions of this ApplyProductionServiceMultiVersionV2Model.  # noqa: E501
        :type: list[ApplyProductionServiceV2Model]
        """
        if self.local_vars_configuration.client_side_validation and service_versions is None:  # noqa: E501
            raise ValueError("Invalid value for `service_versions`, must not be `None`")  # noqa: E501

        self._service_versions = service_versions

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ApplyProductionServiceMultiVersionV2Model):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ApplyProductionServiceMultiVersionV2Model):
            return True

        return self.to_dict() != other.to_dict()



import pathlib
from typing import List, Optional

from rich.console import Console
import yaml

from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.models import (
    AttachMachinePoolToCloudRequest,
    CreateMachinePoolRequest,
    CreateMachinePoolResponse,
    DeleteMachinePoolRequest,
    DescribeMachinePoolRequest,
    DescribeMachinePoolResponse,
    DetachMachinePoolFromCloudRequest,
    ListMachinePoolsResponse,
    UpdateMachinePoolRequest,
)
from anyscale.cloud_utils import get_cloud_id_and_name, get_cloud_resource_id_by_name
from anyscale.controllers.base_controller import BaseController


class MachinePoolController(BaseController):
    def __init__(
        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
    ):
        if log is None:
            log = BlockLogger()

        super().__init__(initialize_auth_api_client=initialize_auth_api_client)
        self.log = log
        self.console = Console()

    def create_machine_pool(self, machine_pool_name: str,) -> CreateMachinePoolResponse:
        response: CreateMachinePoolResponse = self.api_client.create_machine_pool_api_v2_machine_pools_create_post(
            CreateMachinePoolRequest(machine_pool_name=machine_pool_name,)
        ).result
        return response

    def delete_machine_pool(
        self, machine_pool_name: str,
    ):
        self.api_client.delete_machine_pool_api_v2_machine_pools_delete_post(
            DeleteMachinePoolRequest(machine_pool_name=machine_pool_name)
        )

    def update_machine_pool(
        self, machine_pool_name: str, spec_file: str,
    ):
        path = pathlib.Path(spec_file)
        if not path.exists():
            raise FileNotFoundError(f"File {spec_file} does not exist.")

        if not path.is_file():
            raise ValueError(f"File {spec_file} is not a file.")

        spec = yaml.safe_load(path.read_text())

        self.api_client.update_machine_pool_api_v2_machine_pools_update_post(
            UpdateMachinePoolRequest(machine_pool_name=machine_pool_name, spec=spec,)
        )

    def describe_machine_pool(
        self, machine_pool_name: str,
    ) -> DescribeMachinePoolResponse:
        return self.api_client.describe_machine_pool_api_v2_machine_pools_describe_post(
            describe_machine_pool_request=DescribeMachinePoolRequest(
                machine_pool_name=machine_pool_name,
            )
        ).result

    def list_machine_pools(self) -> ListMachinePoolsResponse:
        response = self.api_client.list_machine_pools_api_v2_machine_pools_get().result
        return response

    def format_cloud_and_cloud_resources(
        self, cloud_id: str, cloud_resource_ids: List[str]
    ) -> str:
        # Convert cloud ID and cloud resource IDs to "cloud-name (cloud-resource-name-1, cloud-resource-name-2)".
        _, cloud_name = get_cloud_id_and_name(self.api_client, cloud_id=cloud_id)
        cloud_resource_names = [
            self.api_client.get_cloud_resource_api_v2_clouds_cloud_id_resource_get(
                cloud_id, cloud_resource_id
            ).result.name
            for cloud_resource_id in cloud_resource_ids
        ]
        return f"{cloud_name} ({', '.join(cloud_resource_names)})"

    def attach_machine_pool_to_cloud(
        self,
        machine_pool_name: str,
        cloud_name: str,
        cloud_resource_name: Optional[str] = None,
    ):
        cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_name=cloud_name)

        cloud_resource_id = None
        if cloud_resource_name:
            cloud_resource_id = get_cloud_resource_id_by_name(
                cloud_id=cloud_id,
                cloud_resource_name=cloud_resource_name,
                api_client=self.api_client,
            )

        self.api_client.attach_machine_pool_to_cloud_api_v2_machine_pools_attach_post(
            AttachMachinePoolToCloudRequest(
                machine_pool_name=machine_pool_name,
                cloud_id=cloud_id,
                cloud_resource_id=cloud_resource_id,
            )
        )

    def detach_machine_pool_from_cloud(
        self,
        machine_pool_name: str,
        cloud_name: str,
        cloud_resource_name: Optional[str] = None,
    ):
        cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_name=cloud_name)

        cloud_resource_id = None
        if cloud_resource_name:
            cloud_resource_id = get_cloud_resource_id_by_name(
                cloud_id=cloud_id,
                cloud_resource_name=cloud_resource_name,
                api_client=self.api_client,
            )

        self.api_client.detach_machine_pool_from_cloud_api_v2_machine_pools_detach_post(
            DetachMachinePoolFromCloudRequest(
                machine_pool_name=machine_pool_name,
                cloud_id=cloud_id,
                cloud_resource_id=cloud_resource_id,
            )
        )



from typing import Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.schedule._private.schedule_sdk import PrivateScheduleSDK
from anyscale.schedule.commands import (
    _APPLY_ARG_DOCSTRINGS,
    _APPLY_EXAMPLE,
    _DELETE_ARG_DOCSTRINGS,
    _DELETE_EXAMPLE,
    _LIST_ARG_DOCSTRINGS as _LIST_ARG_DOCSTRINGS,
    _LIST_EXAMPLE as _LIST_EXAMPLE,
    _SET_STATE_ARG_DOCSTRINGS,
    _SET_STATE_EXAMPLE,
    _STATUS_ARG_DOCSTRINGS,
    _STATUS_EXAMPLE,
    _TRIGGER_ARG_DOCSTRINGS,
    _TRIGGER_EXAMPLE,
    _URL_ARG_DOCSTRINGS,
    _URL_EXAMPLE,
    apply as apply,
    delete as delete,
    list as list,  # noqa: A004
    set_state as set_state,
    status as status,
    trigger as trigger,
    url as url,
)
from anyscale.schedule.models import ScheduleConfig, ScheduleState, ScheduleStatus


class ScheduleSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateScheduleSDK(
            client=client, logger=logger, timer=timer
        )

    @sdk_docs(
        doc_py_example=_APPLY_EXAMPLE, arg_docstrings=_APPLY_ARG_DOCSTRINGS,
    )
    def apply(self, config: ScheduleConfig,) -> str:  # noqa: F811
        """Apply or update a schedule.

        Returns the id of the schedule.
        """
        return self._private_sdk.apply(config=config)

    @sdk_docs(
        doc_py_example=_SET_STATE_EXAMPLE, arg_docstrings=_SET_STATE_ARG_DOCSTRINGS,
    )
    def set_state(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        state: ScheduleState,
    ) -> str:  # noqa: F811
        """Set the state of a schedule.

        Returns the id of the schedule.
        """
        return self._private_sdk.set_state(
            id=id, name=name, cloud=cloud, project=project, state=state,
        )

    @sdk_docs(doc_py_example=_STATUS_EXAMPLE, arg_docstrings=_STATUS_ARG_DOCSTRINGS)
    def status(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> ScheduleStatus:
        """Return the status of the schedule.
        """
        return self._private_sdk.status(id=id, name=name, cloud=cloud, project=project)

    @sdk_docs(doc_py_example=_TRIGGER_EXAMPLE, arg_docstrings=_TRIGGER_ARG_DOCSTRINGS)
    def trigger(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        """Trigger the execution of the schedule.
        """
        return self._private_sdk.trigger(id=id, name=name, cloud=cloud, project=project)

    @sdk_docs(doc_py_example=_URL_EXAMPLE, arg_docstrings=_URL_ARG_DOCSTRINGS)
    def url(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        """Get the web UI URL for a schedule."""
        return self._private_sdk.url(id=id, name=name, cloud=cloud, project=project)

    def list(  # noqa: F811 PLR0913
        self,
        *,
        name: Optional[str] = None,
        schedule_id: Optional[str] = None,
        project: Optional[str] = None,
        cloud: Optional[str] = None,
        creator_id: Optional[str] = None,
        include_all_users: bool = False,
        page_size: int = 10,
        max_items: Optional[int] = None,
    ) -> ResultIterator[ScheduleStatus]:
        """List schedules with filtering and pagination.

        Returns a ResultIterator that lazily fetches pages of schedules.

        Args:
            name: Filter by schedule name.
            schedule_id: Fetch a specific schedule by ID.
            project: Filter by project name.
            cloud: Filter by cloud name.
            creator_id: Filter by creator ID.
            include_all_users: Include schedules from all users.
            page_size: Number of items per page.
            max_items: Maximum total items to return.

        Returns:
            ResultIterator of ScheduleStatus objects.
        """
        return self._private_sdk.list(
            name=name,
            schedule_id=schedule_id,
            project=project,
            cloud=cloud,
            creator_id=creator_id,
            include_all_users=include_all_users,
            page_size=page_size,
            max_items=max_items,
        )

    @sdk_docs(doc_py_example=_DELETE_EXAMPLE, arg_docstrings=_DELETE_ARG_DOCSTRINGS)
    def delete(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        """Delete a schedule.

        If the schedule is active, it will be automatically paused before deletion.
        The schedule must have no active triggered jobs.
        Returns the ID of the deleted schedule.
        """
        return self._private_sdk.delete(id=id, name=name, cloud=cloud, project=project)
